local L = IRTLocals;
local f = CreateFrame("Frame");

--Addon vars
local inEncounter = true;
--local spellIDs = {GetSpellInfo(368740), GetSpellInfo(368738)};
local spellIDs = {GetSpellInfo(11426), GetSpellInfo(1459)};
local ticker = nil;
local timer = nil;
local rangeList = {90175, 37727, 8149, 3, 2, 32321, 6450, 21519, 1, 1180, 34471, 32698};

--Leader vars
local leader = "Antmage";
local assigned = {};

--Player vars
local playerName = GetUnitName("player", true);
local oppositeDebuff = nil;
local assignedTo = "Hans";
local assignedMe = nil;

--debug
local printdebug = false;

--Cache
local IRT_UnitBuff = IRT_UnitBuff;
local IRT_Contains = IRT_Contains;
local GetUnitName = GetUnitName;
local IRT_GetRaidLeader = IRT_GetRaidLeader;
local IRT_NotifyPlayer = IRT_NotifyPlayer;
local CheckInteractDistance = CheckInteractDistance;
local IsItemInRange = IsItemInRange;
local UnitIsUnit = UnitIsUnit;
local Ambiguate = Ambiguate;
local UnitIsConnected = UnitIsConnected;
local UnitIsVisible = UnitIsVisible;

f:RegisterEvent("PLAYER_LOGIN");
f:RegisterEvent("ENCOUNTER_START");
f:RegisterEvent("ENCOUNTER_END");
f:RegisterEvent("UNIT_AURA");
f:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
f:RegisterEvent("CHAT_MSG_ADDON");

C_ChatInfo.RegisterAddonMessagePrefix("IRT_LIHU");

function IRT_LIHU_Manual()
	inEncounter = true;
	IRT_GetRaidLeader();
	ticker = nil;
	assigned = {};
	oppositeDebuff = nil;
	assignedTo = nil;
	assignedMe = nil;
	timer = nil;
end

function IRT_LIHU_Debug()
	if (printdebug) then
		printdebug = false;
	else
		printdebug = true;
	end
end

function IRT_LIHU_Force()
	inEncounter = true;
end

function IRT_LihuvimAssign()
	if (assignedTo) then
		if (printdebug) then
			print("assigned to " .. assignedTo .. " already sending to raid to remove")
		end
		C_ChatInfo.SendAddonMessage("IRT_LIHU", playerName, "RAID");
	end
	table.insert(assigned, playerName);
	local partner = false;
	for range, check in ipairs(rangeList) do
		partner = false;
		for j = 1, GetNumGroupMembers() do
			local raider = "raid" .. j;
			if (UnitIsDead(raider) == false and UnitIsVisible(raider) and not UnitIsUnit(raider, "player") and not IRT_Contains(assigned, GetUnitName(raider, true))) then
				if (check == 1 or check == 2 or check == 3) then
					if (CheckInteractDistance(raider, check)) then
						if (IRT_UnitBuff(raider, oppositeDebuff)) then
							partner = GetUnitName(raider, true);
							break;
						end
					end
				else
					if (IsItemInRange(check, raider)) then
						if (IRT_UnitBuff(raider, oppositeDebuff)) then
							partner = GetUnitName(raider, true);
							break;
						end
					end
				end
			end
		end
		if (partner) then
			assignedMe = partner;
			if (printdebug) then
				print("found partner " .. partner);
			end
			IRT_SendAddonCrossRealmMessage("IRT_LIHU", leader, partner);
			break;
		end
	end
end

f:SetScript("OnEvent", function(self, event, ...)
	if (event == "PLAYER_LOGIN") then
		if (IRT_LihuvimEnabled == nil) then IRT_LihuvimEnabled = true; end
	elseif (event == "CHAT_MSG_ADDON" and inEncounter and IRT_LihuvimEnabled) then
		local prefix, msg, channel, sender = ...;
		if (prefix == "IRT_LIHU") then
			local text, target = IRT_DecodeCrossRealmAddonMessage(sender, channel, msg);
			sender = Ambiguate(sender, "none");
			if (channel == "RAID" and target) then
				local shortName, serverName = UnitFullName("player");
				local fullName = shortName .. "-" .. serverName;
				if (UnitIsUnit(target, fullName)) then
				end
			end
			if (channel == "RAID" and target == nil) then
				if (text == "reset") then
					ticker = nil;
					assigned = {};
					assignedTo = nil;
					assignedMe = nil;
					timer = nil;
					if (printdebug) then
						print("reseting all vars");
					end
				else
					if (printdebug) then
						print(text .. " reported they are assigned already");
					end
					table.insert(assigned, text);
					if(assignedMe and UnitIsUnit(text, assignedMe)) then
						if (printdebug) then
							print("had " .. text .. " as assigned to so finding new one");
						end
						IRT_LihuvimAssign();
					end
				end
			elseif (text == "no") then
				if (printdebug) then
					print("got no back added player to assigned and finding new one");
				end
				table.insert(assigned, assignedMe);
				IRT_LihuvimAssign();
			elseif (text == "yes") then
				if (printdebug) then
					print("got yes back on " .. assignedMe);
				end
				ticker = IRT_NotifyPlayer(ticker, assignedMe, "YELL", 5);
				IRT_SendAddonCrossRealmMessage("IRT_LIHU", assignedMe, "assigned");
			elseif (text == "assigned") then
				assignedTo = sender;
				IRT_PopupShow("GO TO " .. Ambiguate(sender, "short"), 5);
			elseif (UnitIsUnit(leader, playerName)) then
				if (timer == nil) then
					timer = C_Timer.After(7, function()
						if (printdebug) then
							print("sending reset to all players");
							C_ChatInfo.SendAddonMessage("IRT_LIHU", "reset", "RAID");
						end
					end)
				end
				if (IRT_Contains(assigned, text)) then
					if (printdebug) then
						print(sender .. " wants to have " .. text .. " but is already assigned, reporting no");
					end
					IRT_SendAddonCrossRealmMessage("IRT_LIHU", sender, "no");
				else
					if (printdebug) then
						print(sender .. " wants to have " .. text .. ", reporting yes");
					end
					table.insert(assigned, text);
					IRT_SendAddonCrossRealmMessage("IRT_LIHU", sender, "yes");
					--C_ChatInfo.SendAddonMessage("IRT_LIHU", "RAID", text);
				end
			end
		end
	elseif (event == "UNIT_AURA" and inEncounter and IRT_LihuvimEnabled) then
		local unit = ...;
		local name = GetUnitName(unit, true);
		if (oppositeDebuff == nil and UnitIsUnit(unit, "player") and IRT_UnitBuff(unit, spellIDs[1])) then
			if (printdebug) then
				print(name .. " has " .. spellIDs[1]);
				print("opposte debuff is " .. spellIDs[2]);
			end
			oppositeDebuff = spellIDs[2];
		elseif (oppositeDebuff == nil and UnitIsUnit(unit, "player") and IRT_UnitBuff(unit, spellIDs[2])) then
			oppositeDebuff = spellIDs[1];
			if (printdebug) then
				print(name .. " has " .. spellIDs[2]);
				print("opposte debuff is " .. spellIDs[1]);
			end
		elseif (oppositeDebuff and UnitIsUnit(unit, "player") and (not IRT_UnitBuff(unit, spellIDs[1]) and not IRT_UnitBuff(unit, spellIDs[2]))) then
			if (printdebug) then
				print(name .. " lost debuff");
				print("opposte debuff removed");
			end
			oppositeDebuff = nil;
		end
	elseif (event == "ENCOUNTER_START" and IRT_LihuvimEnabled) then
		local eID = ...;
		local difficulty = select(3, GetInstanceInfo());
		if (eID == 2539 and difficulty == 16) then
			if (printdebug) then
				print("in encounter");
			end
			inEncounter = true;
			IRT_GetRaidLeader();
			ticker = nil;
			assigned = {};
			oppositeDebuff = nil;
			assignedTo = nil;
			assignedMe = nil;
			timer = nil;
		end
	elseif (event == "ENCOUNTER_END" and inEncounter and IRT_LihuvimEnabled) then
		inEncounter = false;
		ticker = nil;
		assigned = {};
		oppositeDebuff = nil;
		assignedTo = nil;
		assignedMe = nil;
		timer = nil;
		IRT_PopupHide(L.BOSS_FILE);
	end
end);